package net.sf.seaf.persistence;

import java.io.Serializable;
import java.util.List;

/**
 * Generic DAO interface.
 * 
 * @param <E>
 *            The entity
 * @param <ID>
 *            The primary key class
 * @param <EC>
 *            The bean of entity criteria
 * @param <P>
 *            Property enum
 */
public interface GenericDAO<E, ID extends Serializable, EC extends CriteriaBean<P>, P extends Enum<?>> {

	/**
	 * Create an instance of the entity. Makes a transient instance persistent.
	 * 
	 * @param entity
	 *            Transient instance of the entity
	 * @return Entity Persistent instance created
	 */
	E create(E entity);

	/**
	 * Retrieve an instance of the entity by its primary key. The instance must
	 * exists.
	 * 
	 * @param id
	 *            The value of the primary key
	 * @return Entity Persistent instance retrieved
	 */
	E retrieve(ID id);

	/**
	 * Retrieve an instance of the entity by its primary key, if it exists.
	 * Returns null if it does not exist.
	 * 
	 * @param id
	 *            The value of the primary key
	 * @return Entity Persistent instance retrieved or null
	 */
	E retrieveIfExists(ID id);

	/**
	 * Update an instance of the entity.
	 * 
	 * @param entity
	 *            A detached instance to be updated
	 * @return Entity The updated persistent instance
	 */
	E update(E entity);

	/**
	 * Copy the state of the given object onto the persistent object with the
	 * same identifier.
	 * 
	 * @param entity
	 *            A detached instance to merge with the corresponding persistent
	 *            instance
	 * @return Entity The updated, registered persistent instance
	 */
	E merge(E entity);

	/**
	 * Delete an instance of the entity.
	 * 
	 * @param entity
	 *            A persistent or detached instance to be deleted
	 */
	void delete(E entity);

	/**
	 * Create or update an instance of the entity, depending on whether the
	 * passed in instance is transient or detached.
	 * 
	 * @param entity
	 *            An entity instance to be created or updated
	 * @return Entity The created or updated persistent instance
	 */
	E createOrUpdate(E entity);

	/**
	 * Find instances of the entity based on the given entity criteria.
	 * 
	 * @param criteria
	 *            The entity criteria
	 * @return List<Entity> The instances of the entity found matching the
	 *         entity criteria
	 */
	List<E> find(EC criteria);

	/**
	 * Find instances of the entity based on the given entity criteria and
	 * return a specified subset of the matching records.
	 * 
	 * @param criteria
	 *            The entity criteria
	 * @param fromRecNo
	 *            The 1-based starting index of the first record to return
	 * @param count
	 *            The number of records to return
	 * @return List<Entity> The instances of the entity found matching the
	 *         entity criteria
	 */
	List<E> find(EC criteria, int fromRecNo, int count);

	/**
	 * Return the number of entities matching the given entity criteria.
	 * 
	 * @param criteria
	 *            The entity criteria
	 * @return int The number of matching entity instances
	 */
	int size(EC criteria);

	/**
	 * Flush the persistence context to underlying persistence store.
	 */
	void flush();

}
